/* utrb.h, M. Welsh (mdw@cs.cornell.edu)
 *
 * Definitions for UTlb module exported functions.
 */

// Soft TLB
typedef struct {
	ULONG page_id; // virtual address and endpt id
	ULONG paddr;   // physical address
	ULONG saddr;   // system-mapped virtual address
	ULONG age;     // age
	UCHAR refcnt;  // entry cannot be unpinned until refcnt == 0
	union {        // the MDL (statically-allocated)
		MDL mdl;
		unsigned char mdlbuf[sizeof(MDL) + sizeof(ULONG)];
	} u;
} UNET_STLB, *PUNET_STLB;

#define UNET_STLB_SIZE 256
#define UNET_STLB_SIZE_LOG 8
#define UNET_STLB_PROT_RO 0 // Read-only 
#define UNET_STLB_PROT_RW 1 // Read-write

#define UNET_STLB_HASH(eid, vaddr) \
   ((((eid) & 0x03) << UNET_STLB_SIZE_LOG-2) | \
		(((vaddr) >> PAGE_SHIFT) & ((1 << UNET_STLB_SIZE_LOG-2)-1)))

#define PAGE_ADDR(addr) ((addr) & ~(PAGE_SIZE-1))
#define PAGE_OFFSET(addr) ((addr) & (PAGE_SIZE-1))

// STLB victim cache. This has the special property that if entry->refcnt
// is non-zero, we don't throw an entry out of the victim cache; instead
// we allow it to remain and the page stays pinned.
typedef struct _UNET_STLB_VCACHE {
	UNET_STLB entry;
	struct _UNET_STLB_VCACHE *next;
} UNET_STLB_VCACHE, *PUNET_STLB_VCACHE;
	
#define UNET_STLB_VCACHE_SIZE 4 // Target size for victim cache

ULONG StlbTranslate(UNET_ENDPOINT_ID eid,
										ULONG vaddr, int prot, BOOLEAN handle_miss,
										PULONG system_addr);
void StlbClear(UNET_ENDPOINT_ID eid);
void StlbDecRefcnt(UNET_ENDPOINT_ID eid, ULONG vaddr, ULONG paddr);
