/* libunet.h, M. Welsh (mdw@cs.cornell.edu)
 * 
 * Interface to 'libunet', the demonstration U-Net library.
 * Modified by mdw for NT4.0, 23-9-97, from libunet.h v1.7
 */

/*
 * Copyright (c) 1995 by Cornell University.
 * All rights reserved.
 *
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 *
 * IN NO EVENT SHALL CORNELL UNIVERSITY BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
 * IF CORNELL UNIVERSITY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * CORNELL UNIVERSITY SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND CORNELL UNIVERSITY HAS NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 *
 * $Id: libunet.h,v 1.7 1996/04/01 02:03:35 mdw Exp $
 */


#ifndef _LIBUNET_H
#define _LIBUNET_H

#include <stdio.h>

#include "unetuser.h"

#define MIN(a,b) ((a)<(b)?(a):(b))
#define MAX(a,b) ((a)>(b)?(a):(b))

/* Some UNIX types NT is missing */
typedef unsigned char * caddr_t;
typedef unsigned int u_int;
struct  iovec {
        caddr_t iov_base;
        int     iov_len;
};

struct unet_endpt_user {

  /* User address-space pointers into the communication segment */
  UNET_TX_BUFFER_DESCRIPTOR *tx_first;
  volatile UNET_TX_BUFFER_DESCRIPTOR *tx_next;
  UNET_TX_BUFFER_DESCRIPTOR *tx_last;
  volatile unsigned int *tx_wall;
  UNET_TX_BUFFER_DESCRIPTOR *tx_wall_cache;

  UNET_RECV_BUFFER_DESCRIPTOR *free_first;
  volatile UNET_RECV_BUFFER_DESCRIPTOR *free_next;
  UNET_RECV_BUFFER_DESCRIPTOR *free_last;
  volatile unsigned int *free_wall;

  UNET_RECEIVE_DESCRIPTOR *rx_first;
  volatile UNET_RECEIVE_DESCRIPTOR *rx_next;
  UNET_RECEIVE_DESCRIPTOR *rx_last;
  volatile unsigned int *rx_wall;

  caddr_t  buffer_area;
  caddr_t  rxb_area;
  caddr_t  txb_area;
  u_int    rxb_numbuff;
  u_int    rxb_buffsize_alloc, rxb_buffsize_used;

  HANDLE hDriver; /* Driver handle */
  UNET_ENDPOINT_ID endpointId;  /* Endpoint ID */
  UNET_INFO_RESPONSE info;  /* Information from device */
  int rxwall_deferred;    /* Rx wall needs to be updated due to inline msg */
  UNET_RECEIVE_DESCRIPTOR *rxwall_deferred_entry; 

};


/* Define to MHz speed of Pentium/P6 processor (used for accurate timings
 * using RDTSC instruction). If non-Pentium undefine this.
 */
#define PENTIUM_MHZ 133.0
extern double get_seconds(void);

extern int str_to_device(char *str, UNET_DEV_TYPE *devtype, UNET_DEV_INDEX *devindex);
extern int str_to_addr(UNET_DEV_TYPE devtype, char *str, UNET_CHANNEL_ADDRESS *addr);
extern HANDLE unet_opendev(UNET_DEV_TYPE devtype, UNET_DEV_INDEX devindex,
			   UNET_INFO_RESPONSE *info);
extern struct unet_endpt_user *unet_create_endpt(HANDLE hDriver,
						 UNET_INFO_RESPONSE *info,
						 UNET_CREATE_ENDPOINT_REQUEST *create,
						 int num_rx_buffers);
extern int unet_destroy_endpt(HANDLE hDriver, UNET_ENDPOINT_ID endpointId);
extern UNET_CHANNEL_ID unet_activate_chan(HANDLE hDriver, UNET_ACTIVATE_CHANNEL_REQUEST *activate);
extern int unet_deactivate_chan(HANDLE hDriver, UNET_DEACTIVATE_CHANNEL_REQUEST *deactivate);
extern void unet_update_txwall(struct unet_endpt_user *endpt);
extern int unet_sendv(struct unet_endpt_user *endpt, UNET_CHANNEL_ID chan, struct iovec *iov, int iovcnt);
extern int unet_send(struct unet_endpt_user *endpt, UNET_CHANNEL_ID chan, unsigned char *buf, int len);
extern int unet_recv_poll(struct unet_endpt_user *endpt);
extern int unet_recv(struct unet_endpt_user *endpt, short *chan, struct iovec *iov, int *iovcnt);
extern void unet_free_rxbuf(struct unet_endpt_user *endpt, unsigned int addr);
extern void unet_free_rxv(struct unet_endpt_user *endpt, struct iovec *iov, int iovc);

extern void Perror(char *s);

#if 0 /* XXX mdw (23-9-96) Unimplemented for NT4.0 */
extern int unet_intr(int fd, void(*handler)());
extern int unet_notify(int fd, struct unet_intr_ctrl *intrctrl);
#endif

#endif /* _LIBUNET_H */

